/* File: CLIENT.C */

#include <stdlib.h>
#include <string.h>
#include <malloc.h>
#include "CLIENT.h"
#include "res.h"
#include "fi.h"
#include "RES_DBOX.h"
#include "VAR_DBOX.h"
#include "ERROR.h"
#include "OUTPUT.h"
#include "const.h"
#include "BAYES1.h"
#include "SHIT.h"
#include "MIG1.h"
#include "UNT.h"
#include "EXKOR.h"
#include "LBAYES.h"
#include "LPMIN.h"
#include "GLOPT.h"
#include "FLEXI.h"
#include "HELP.h"

#define DBOX_BACKGROUND WhitePixel (display, DefaultScreen (display))
#define DBOX_FOREGROUND BlackPixel (display, DefaultScreen (display))

void CLIENT::Run (void)
{
   enum {Nothing, Bayes1, Mig1, Unt, Exkor, Glopt, Lpmin, Nlp, Flexi, LBayes} met;
   double *xl, *xu, *x;
   int    it      = 0;
   double y;
   double *fm  = NULL;
   double **xn = NULL;

   met = Nothing;
   x   = (double *) malloc (number_of_variables * sizeof (double));
   xl  = (double *) malloc (number_of_variables * sizeof (double));
   xu  = (double *) malloc (number_of_variables * sizeof (double));
   if (x == NULL || xl == NULL || xu == NULL) {
      ERROR ("no memory");
   }
   int i;
   for (i = 0; i < number_of_variables; i++) {
      xl [i] = -1.;
      xu [i] = 1.;
      x  [i] = 0.;
   }

   METHODBAYES1 MethodBayes1 (xl, xu, number_of_variables);
   METHODLPMIN  MethodLpmin (xl, xu, number_of_variables);
   METHODSHIT   MethodNlp (x, xl, xu, number_of_variables);
   METHODMIG1   MethodMig1 (xl, xu, number_of_variables);
   METHODUNT    MethodUnt (xl, xu, number_of_variables);
   METHODEXKOR  MethodExkor (x, xl, xu, number_of_variables);
   METHODLBAYES MethodLBayes (x, xl, xu, number_of_variables);
   METHODFLEXI  MethodFlexi (x, xl, xu, number_of_variables);
   METHODGLOPT  MethodGlopt (xl, xu, number_of_variables);

   HELP Help ( display, this, "help.res" );

   output .Init (number_of_variables, display, xl, xu, (WINDOW *) this, 
      GetFont (), &it, &xn, &fm);
   for (;;) {
      XEvent event;

      XNextEvent (display, &event);

      int done = Dispatch (&event);

      if (done == NOTHING) continue;
      if (done == QUIT) {
         output .Finish ();
         if (fm != NULL) {
            for (i = 0; i < it; i++)
               free ((char *) xn [i]);
            free ((char *) xn);
            free ((char *) fm);
         }
         free ((char *) xu);
         free ((char *) xl);
         free ((char *) x);
         break;
      }
      switch (done) {
      case BAYES1:
	 if (fm != NULL) {
//fprintf (stderr, "CLIENT: freeing\n");
	    for (i = 0; i < it; i++) free ((char *) xn [i]);
	    free ((char *) xn);
	    free ((char *) fm);
            fm = NULL;
            xn = NULL;
	 }
	 it  = 0;
         met = Bayes1;
         break;
      case NLP:
	 if (fm != NULL) {
	    for (i = 0; i < it; i++) free ((char *) xn [i]);
	    free ((char *) xn);
	    free ((char *) fm);
            fm = NULL;
            xn = NULL;
	 }
	 it  = 0;
         met = Nlp;
         break;
      case MIG1:
	 if (fm != NULL) {
	    for (i = 0; i < it; i++) free ((char *) xn [i]);
	    free ((char *) xn);
	    free ((char *) fm);
            fm = NULL;
            xn = NULL;
	 }
	 it  = 0;
         met = Mig1;
         break;
      case UNT:
	 if (fm != NULL) {
	    for (i = 0; i < it; i++) free ((char *) xn [i]);
	    free ((char *) xn);
	    free ((char *) fm);
            fm = NULL;
            xn = NULL;
	 }
	 it  = 0;
         met = Unt;
         break;
      case EXKOR:
	 if (fm != NULL) {
	    for (i = 0; i < it; i++) free ((char *) xn [i]);
	    free ((char *) xn);
	    free ((char *) fm);
            fm = NULL;
            xn = NULL;
	 }
	 it  = 0;
         met = Exkor;
         break;
      case GLOPT:
	 if (fm != NULL) {
	    for (i = 0; i < it; i++) free ((char *) xn [i]);
	    free ((char *) xn);
	    free ((char *) fm);
            fm = NULL;
            xn = NULL;
	 }
	 it  = 0;
         met = Glopt;
         break;
      case LPMIN:
	 if (fm != NULL) {
	    for (i = 0; i < it; i++) free ((char *) xn [i]);
	    free ((char *) xn);
	    free ((char *) fm);
            fm = NULL;
            xn = NULL;
	 }
	 it  = 0;
         met = Lpmin;
         break;
      case FLEXI:
	 if (fm != NULL) {
	    for (i = 0; i < it; i++) free ((char *) xn [i]);
	    free ((char *) xn);
	    free ((char *) fm);
            fm = NULL;
            xn = NULL;
	 }
	 it  = 0;
         met = Flexi;
         break;
      case LBAYES:
	 if (fm != NULL) {
	    for (i = 0; i < it; i++) free ((char *) xn [i]);
	    free ((char *) xn);
	    free ((char *) fm);
            fm = NULL;
            xn = NULL;
	 }
	 it  = 0;
         met = LBayes;
         break;
      case PARAMETERS:
	 if (fm != NULL) {
//fprintf (stderr, "CLIENT: parameters has changed: freeing\n");
	    for (i = 0; i < it; i++) free ((char *) xn [i]);
	    free ((char *) xn);
	    free ((char *) fm);
            fm = NULL;
            xn = NULL;
	 }
         switch (met) {
         case Bayes1:
            it = MethodBayes1 .EnterParameters (display,this);
            break;
         case Lpmin:
            it = MethodLpmin .EnterParameters (display,this);
            break;
         case Nlp:
            it = MethodNlp .EnterParameters (display, this);
            break;
         case Mig1:
            it = MethodMig1 .EnterParameters (display, this);
            break;
         case Unt:
            it = MethodUnt .EnterParameters (display, this);
            break;
         case Exkor:
            it = MethodExkor .EnterParameters (display, this);
            break;
         case LBayes:
            it = MethodLBayes .EnterParameters (display, this);
            break;
         case Glopt:
            it = MethodGlopt .EnterParameters (display, this);
            break;
         case Flexi:
            it = MethodFlexi .EnterParameters (display, this);
            break;
	 case Nothing:
	    break;
         }
         break;
      case RUN:
	 if (met != Nothing && it > 0) {
	    switch (met) {
	    case Bayes1:
	    case Lpmin:
	    case Mig1:
	    case Unt:
	    case Exkor:
	    case Glopt:
	    case Nlp:
	    case LBayes:
	    case Flexi:
	       {
//fprintf (stderr, "CLIENT: running: allocating\n");
		  fm = (double *) malloc (it * sizeof (double));
		  xn = (double **) malloc (it * sizeof (double *));
		  if (fm == NULL || xn == NULL) {
		     ERROR ("no memory");
		  }
		  for (i = 0; i < it; i++) {
		     if ((xn [i] = (double *) malloc (number_of_variables * 
			sizeof (double))) == NULL) {
			ERROR ("no memory");
		     }
                  }
		  output .SetN (0);
		  output .DisplayItself ();
		  switch (met) {
		  case Bayes1:
		     MethodBayes1 .Run (x, number_of_variables, &y, fi, fm, xn);
		     break;
		  case Glopt:
		     MethodGlopt .Run (x, number_of_variables, &y, fi, fm, xn);
		     break;
		  case Lpmin:
		     MethodLpmin .Run (x, number_of_variables, &y, fi, fm, xn);
		     break;
		  case Mig1:
		     MethodMig1 .Run (x, number_of_variables, &y, fi, fm, xn);
		     break;
		  case Unt:
		     MethodUnt .Run (x, number_of_variables, &y, fi, fm, xn);
		     break;
		  case Exkor:
		     MethodExkor .Run (x, number_of_variables, &y, fi, fm, xn);
		     break;
		  case Nlp:
		     MethodNlp .Run (x, number_of_variables, &y, fi, constr, fm, 
			xn);
		     break;
		  case LBayes:
		     MethodLBayes .Run (x, number_of_variables, &y, fi, fm, xn);
		     break;
		  case Flexi:
		     MethodFlexi .Run (x, number_of_variables, &y, fi, constr, 
                        fm, xn);
		     break;
		  case Nothing:
		     break;
		  }
	       }                        
	       break;
	    case Nothing:
	       break;
	    }
	 }
	 break;
      case RESULTS:
	 if (met != Nothing) {
	    switch (met) {
	    case Bayes1:
	    case Mig1:
	    case Nlp:
	    case Unt:
	    case Exkor:
	    case LBayes:
	    case Lpmin:
	    case Glopt:
	    case Flexi:
	       {
		  VELEMENT Variable [2];

		  Variable [0] .ident   = X;
		  Variable [0] .size    = number_of_variables;
		  Variable [0] .pointer = (void *) x;
		  Variable [0] .format  = DOUBLE;
		  Variable [1] .pointer = NULL;

		  int VPar [1];

		  VPar [0] = Variable [0] .size;

		  int VID [1];

		  VID [0] = Variable [0] .ident;

		  FELEMENT Fixed [2];

		  Fixed [0] .ident   = Y;
		  Fixed [0] .pointer = (void *) &y;
		  Fixed [0] .format  = DOUBLE;
		  Fixed [1] .pointer = NULL;

		  RES_DBOX *dbox = new RES_DBOX (display, this, DBOX_FOREGROUND,
                     DBOX_BACKGROUND, "res.res", Fixed, Variable, VPar, VID);

		  if (dbox == NULL) {
		     ERROR ("no memory");
		  }
		  dbox ->Dialog ();
		  delete dbox;
	       }
	       break;
	    case Nothing:
	       break;
	    }
	 }
         break;
      case PROJECTION:
         {
            int      var   = 1;
            FELEMENT Fixed [2];

            Fixed [0] .ident   = VAR;
            Fixed [0] .pointer = (void *) &var;
            Fixed [0] .format  = INT;
            Fixed [1] .pointer = NULL;

            VELEMENT Variable [1];

            Variable [0] .pointer = NULL;

            VAR_DBOX *dbox = new VAR_DBOX (display, this, DBOX_FOREGROUND,
                  DBOX_BACKGROUND, "var.res", Fixed, Variable);

            if (dbox == NULL) {
               ERROR ("no memory");
            }
            for (;;) {
               dbox ->Dialog ();
               if (var > 0 && var <= number_of_variables) break;
            }
            delete dbox;
            var--;
            if (output .IsProjectionWindow (var) == False) {
               output .CreateProjectionWindow (var);
            } else {
               output .DestroyProjectionWindow (var);
	    }
         }
         break;
      case CONVERGENCE:
	 if (output .IsConvergenceWindow () == False) {
	    output .CreateConvergenceWindow ();
	 } else {
            output .DestroyConvergenceWindow ();
	 }
         break;
      case NUMERIC:
	 if (output .IsNumericWindow () == False)  {
	    output .CreateNumericWindow (number_of_variables);
	 } else {
	    output .DestroyNumericWindow ();
	 }
         break;
      case H_BAYES1 : Help.ShowItem(H_Bayes1) ; break ;
      case H_MIG1   : Help.ShowItem(H_Mig1) ; break ;
      case H_UNT    : Help.ShowItem(H_Unt) ; break ;
      case H_EXKOR  : Help.ShowItem(H_Exkor) ; break ;
      case H_GLOPT  : Help.ShowItem(H_Glopt) ; break ;
      case H_LPMIN  : Help.ShowItem(H_Lpmin) ; break ;
      case H_NLP    : Help.ShowItem(H_Nlp) ; break ;
      case H_FLEXI  : Help.ShowItem(H_Flexi) ; break ;
      case H_LBAYES : Help.ShowItem(H_Lbayes) ; break ;
      }
   }
}

void CLIENT::Resize (void)
{
   XWindowAttributes pxwa, xwa;

   parent ->GetAttributes (&pxwa);
   GetAttributes (&xwa);
   WINDOW::Resize (pxwa .width, pxwa .height - xwa .y);
}

CLIENT::CLIENT (Display *dsp, WINDOW *prnt, int x, int y, unsigned int width,
   unsigned int height, unsigned long foreground, unsigned long background) :
   WINDOW (dsp, prnt, CLIENT_NAME, NULL, x, y, width, height, 0, 0,
   background, True, prnt ->GetFont (), foreground)
{
   SetInput (ButtonPressMask);
   Lower ();
}

int CLIENT::Go (XEvent *event)
{
   switch (event ->type) {
   default:
      break;
   }
   return NOTHING;
}

