/* File: HELP.C */

#include "HELP.h"

#include "const.h"
#include "work.h"
#include "res.h"
#include "ERROR.h"

#define HELP_BACKGROUND WhitePixel (dsp, DefaultScreen (dsp))
#define HELP_FOREGROUND BlackPixel (dsp, DefaultScreen (dsp))

#define INTERVAL 4


void HELP_ITEM::S1 ( WINDOW* win, FILE *in)
{
   XFontStruct *font_info = win->QueryFont ();

   width = height = 0 ;

   for (; nlines < MAX_LINE_NUM ; nlines++) {
      char token [128];

      read_token (in, token);
      if (feof (in)) break;
      if (strcmp (token, "TEXT") == 0) {
         read_text (in, token);

         width = MAX (width, TEXT_WIDTH (token));

	 lines [nlines] = new char [strlen(token)+1] ;
         if ( lines [nlines] == NULL )
             ERROR ("no memory") ;

         strcpy ( lines [nlines], token ) ;

      } else 
         break;
   }
   height += nlines * CHAR_HEIGHT ;

   XFreeFontInfo (NULL, font_info, 1) ;
}


HELP_ITEM::HELP_ITEM () 
{

    for ( int i = 0 ; i < MAX_LINE_NUM ; i++ ) 
        lines [i] = 0 ;
    nlines = width = height = 0 ;
}


HELP_ITEM::~HELP_ITEM ()
{           
    for ( int i = 0 ; i < nlines ; i++ )
	delete lines[i] ;
}


void HELP::ShowItem(HelpItem it) 
{
    HELP_ITEM &item = items [current = it] ;

    if ( item.nlines == 0 ) return ;

    XWindowAttributes xwa ; 
    button -> GetAttributes (&xwa) ;
    
    XWindowAttributes pxwa ; 
    parent -> GetAttributes (&pxwa) ;

    unsigned w = item.width + xwa.width + 2 * INTERVAL ,
	     h = item.height + xwa.height + 3 * INTERVAL ,
	     x = (pxwa.width > w) ? (pxwa.width - w)/2 : 0 ,
	     y = (pxwa.height > h) ? (pxwa.height - h)/2 : 0 ;

    MoveResize( x, y, w, h ) ;

    button -> Move ( item.width/2 + INTERVAL , item.height + 2 * INTERVAL ) ;

    Clear() ;
    SetVisible ( True) ;
    Show ( True) ;
    Draw (item) ;

    for (;;) {
       XEvent event;
	
       XNextEvent (display, &event);
       if (Dispatch (&event) == EXIT_HELP) break;
    }
    Show (False);
    XSync (display, True);
}


void HELP::Draw(HELP_ITEM& item) 
{
    XFontStruct *font_info = QueryFont ();
    int char_height = CHAR_HEIGHT ;
    XFreeFontInfo (NULL, font_info, 1) ;

    int y = char_height - 2 + INTERVAL ;
    for (int i = 0 ; i < item.nlines ; i++ , y+= char_height) 
	DrawString ( INTERVAL, y, item.lines[i]) ;
}


const char *OK_LABEL = "OK" ;

HELP::HELP (Display *dsp, WINDOW *prnt, char *name) :
   WINDOW (dsp, prnt, HELP_NAME, NULL, 1, 1, 1, 1, 1,
   BlackPixel (dsp, DefaultScreen (dsp)), 
   HELP_BACKGROUND, True, prnt ->GetFont (), HELP_FOREGROUND)
{
   XFontStruct *font_info = QueryFont ();

   Load (name);
   button = new BUTTON ( dsp, this, (char *)OK_LABEL, 1, 1,
                (unsigned int)(TEXT_WIDTH (OK_LABEL) + 2),
                 (unsigned int)(CHAR_HEIGHT + 2), OK ) ;
   SetInput (ExposureMask) ;
   XFreeFontInfo (NULL, font_info, 1) ;
}


void HELP::S0 (FILE *in)
{
   int x = 0;

   for (;;) {
      char token [128];
      HelpItem help_item ;

      read_token (in, token);
      if (feof (in)) break;
      if (strcmp (token, "HELPITEM") == 0) {
         switch (read_index (in)) {
	     case H_BAYES1 : help_item = H_Bayes1 ; break ;
	     case H_MIG1 :   help_item = H_Mig1 ; break ;
	     case H_UNT :    help_item = H_Unt ; break ;
	     case H_EXKOR :  help_item = H_Exkor ; break ;
	     case H_GLOPT :  help_item = H_Glopt ; break ;
	     case H_LPMIN :  help_item = H_Lpmin ; break ;
	     case H_NLP :    help_item = H_Nlp ; break ;
	     case H_FLEXI :  help_item = H_Flexi ; break ;
	     case H_LBAYES : help_item = H_Lbayes ; break ;
	 }
      
         read_token (in, token);

         items [help_item] . S1 ( this, in);
      } else
         break;
   }
}

int HELP::Go (XEvent *event)
{
   int id;
   switch (event ->type) {
   case ClientMessage:
         id = (int) event ->xclient .data .l [1];
         if (id == OK) return EXIT_HELP;
   case Expose:
      if (event ->xexpose .count == 0) Draw (items[current]);
      break;
   }
   return NOTHING;
}


void HELP::Load (char *name)
{
   FILE *in = fopen (name, "r");

   if (in == NULL) {
      ERROR ("HELP");
   }
   S0 (in);
   fclose (in);
}
