/* File: MENU_ITEM.cc */

#include <stdio.h>
#include "const.h"
#include "MENU_ITEM.h"
#include "SUBMENU.h"

void MENU_ITEM::Draw (void)
{
   XWindowAttributes xwa;

   GetAttributes (&xwa);

   char *name = GetName ();

   DrawString (2, xwa .height - 2, name);
   XFree (name);
}

void MENU_ITEM::Reverse (void)
{
   unsigned long foreground = GetForeground ();
   unsigned long background = GetBackground ();

   SetBackground (foreground);

   XWindowAttributes xwa;

   GetAttributes (&xwa);
   DrawRectangle (0, 0, xwa .width, xwa .height);
   FillRectangle (0, 0, xwa .width, xwa .height);
   SetForeground (background);
   Draw ();
}

int MENU_ITEM::Go (XEvent *event)
{
   switch (event ->type) {
   case Expose:
      if (event ->xexpose .count == 0) Draw ();
      break;
   case ButtonRelease:
      Reverse ();
      if (submenu) {
         submenu ->Show (False);
         submenu ->SetVisible (False);
      } else
         return index;
      break;
   case ButtonPress:
      XUngrabPointer (event ->xbutton .display, CurrentTime);
      if (submenu) {
         submenu ->SetVisible (True);
         submenu ->Show (True);
      }
      Reverse ();
      break;
   case EnterNotify:
      if (event ->xcrossing .state) {
         Reverse ();
         if (submenu) {
            submenu ->SetVisible (True);
            submenu ->Show (True);
         }
      }
      break;
   case LeaveNotify:
      if (event ->xcrossing .state) {
         Reverse ();

         XWindowAttributes xwa;

         GetAttributes (&xwa);
         if (submenu && (event ->xcrossing .x <= 0 ||
            event ->xcrossing .x >= xwa .width ||
            event ->xcrossing .y <= 0)) {
            submenu ->Show (False);
            submenu ->SetVisible (False);
         }
      }
      break;
   }
   return NOTHING;
}

MENU_ITEM::MENU_ITEM (Display *dsp, WINDOW *prnt, char *Name, int x, int y,
   unsigned int width, unsigned int height, SUBMENU *Submenu, int Index) :
   WINDOW (dsp, prnt, MENU_ITEM_NAME, Name, x, y, width, height, 0, 0,
      prnt ->GetBackground (), True, prnt ->GetFont (), 
      prnt ->GetForeground ())
{
   submenu = Submenu;
   index   = Index;
   SetInput (ButtonPressMask | ButtonReleaseMask | ExposureMask |
      EnterWindowMask | LeaveWindowMask);
}

MENU_ITEM::~MENU_ITEM (void)
{
   if (submenu) delete submenu;
}
