/* File: OUTPUT.C */

#define __OUTPUT
#include <malloc.h>
#include "const.h"
#include "OUTPUT.h"
#include "VAR_DBOX.h"
#include "res.h"
#include "ERROR.h"

#define DBOX_BACKGROUND WhitePixel (output .display, \
           DefaultScreen (output .display))
#define DBOX_FOREGROUND BlackPixel (output .display, \
           DefaultScreen (output .display))
#define CONV_FOREGROUND BlackPixel (display, DefaultScreen (display))
#define CONV_BACKGROUND WhitePixel (display, DefaultScreen (display))
#define CONV_BORDER     BlackPixel (display, DefaultScreen (display))
#define PROJ_FOREGROUND BlackPixel (display, DefaultScreen (display))
#define PROJ_BACKGROUND WhitePixel (display, DefaultScreen (display))
#define NUM_FOREGROUND  BlackPixel (display, DefaultScreen (display))
#define NUM_BACKGROUND  WhitePixel (display, DefaultScreen (display))
#define PROJ_BORDER     BlackPixel (display, DefaultScreen (display))

void OUTPUT::Init (int n, Display *dsp, double *A, double *B, WINDOW *Area,
   Font Fid, int *It, double ***Xn, double **Fm)
{
   fid             = Fid;
   area            = Area;
   min_bound       = A;
   max_bound       = B;
   display         = dsp;
   window          = NULL;
   nwindow         = NULL;
   fm              = Fm;
   xn              = Xn;
   max_n_of_points = It;
   pwindow        = (PROJ_WINDOW **) malloc (n * sizeof (PROJ_WINDOW *));
   if (pwindow == NULL) {
      ERROR ("no memory");
   }
   n_of_variables = n;
   for (int i = 0; i < n; i++)
      pwindow [i] = NULL;
}

void OUTPUT::Finish (void)
{
   if (IsConvergenceWindow () == True) DestroyConvergenceWindow ();
   if (IsNumericWindow () == True) DestroyNumericWindow ();
   for (int i = 0; i < n_of_variables; i++)
      if (IsProjectionWindow (i) == True) DestroyProjectionWindow (i);
   free ((char *) pwindow);
}

Bool OUTPUT::IsConvergenceWindow (void)
{
   if (window != NULL) return True;
   return False;
}

Bool OUTPUT::IsNumericWindow (void)
{
   if (nwindow != NULL) return True;
   return False;
}

Bool OUTPUT::IsProjectionWindow (int var)
{
   if (pwindow [var] != NULL) return True;
   return False;
}

void OUTPUT::CreateConvergenceWindow (void)
{
   window = new CONV_WINDOW (display, NULL, CONV_WINDOW_NAME, "Convergence", 
      0, 0, 200, 200, 5, CONV_BORDER, CONV_BACKGROUND, CONV_FOREGROUND, fid);
   if (window == NULL) {
      ERROR ("no memory");
   }
//   if (fm != NULL) {
      window ->SetN (n_of_points);
      window ->SetMaxN (max_n_of_points);
      window ->SetFm (fm);
//   }
   window ->Show (True);
}

void OUTPUT::CreateNumericWindow (int n)
{
   int VPar [1];

   VPar [0] = n;

   int VID [1];

   VID [0] = X;

   nwindow = new NUM_WINDOW (display, NULL, NUM_FOREGROUND,
      NUM_BACKGROUND, "num.res", VPar, VID, "Numeric", 5, fid);
   if (nwindow == NULL) {
      ERROR ("no memory");
   }
   nwindow ->Show (True);
}

void OUTPUT::CreateProjectionWindow (int var)
{
   char buf [128];
//fprintf (stderr, "creating projection window\n");
   sprintf (buf, "Projection:%.2d", var + 1);
   pwindow [var] = new PROJ_WINDOW (display, NULL, PROJ_WINDOW_NAME, buf, 
      0, 0, 200, 200, 5, PROJ_BORDER, PROJ_BACKGROUND, PROJ_FOREGROUND, fid);
   if (pwindow [var] == NULL) {
      ERROR ("no memory");
   }
   pwindow [var] ->SetNOfVariables (n_of_variables);
   pwindow [var] ->SetVariable (var);
   pwindow [var] ->SetMinBound (min_bound);
   pwindow [var] ->SetMaxBound (max_bound);
//   if (fm != NULL) {
      pwindow [var] ->SetN (n_of_points);
      pwindow [var] ->SetMaxN (max_n_of_points);
      pwindow [var] ->SetFm (fm);
      pwindow [var] ->SetXn (xn);
//   }
   pwindow [var] ->Show (True);
//fprintf (stderr, "projection window is shown %p\n", pwindow [var]);
}

void OUTPUT::DestroyProjectionWindow (int var)
{
   pwindow [var] ->Show (False);
   delete pwindow [var];
   pwindow [var] = NULL;
}

void OUTPUT::DestroyConvergenceWindow (void)
{
   window ->Show (False);
   delete window;
   window = NULL;
}

void OUTPUT::DestroyNumericWindow (void)
{
   nwindow ->Show (False);
   delete nwindow;
   nwindow = NULL;
}

void OUTPUT::SetN (int n)
{
   n_of_points = n;
   if (window != NULL) window ->SetN (n);
   for (int i = 0; i < n_of_variables; i++)
      if (pwindow [i] != NULL) pwindow [i] ->SetN (n);
   if (nwindow != NULL) nwindow ->SetPoint (0, NULL);
}

void OUTPUT::DisplayItself (void)
{
   if (nwindow != NULL) nwindow ->DisplayItself ();
   if (window != NULL) window ->DisplayItself ();
   for (int i = 0; i < n_of_variables; i++)
      if (pwindow [i] != NULL) pwindow [i] ->DisplayItself ();
}

void OUTPUT::SetMinBound (double *A)
{
   min_bound = A;
   for (int i = 0; i < n_of_variables; i++)
      if (pwindow [i] != NULL) pwindow [i] ->SetMinBound (A);
}

void OUTPUT::SetMaxBound (double *B)
{
   max_bound = B;
   for (int i = 0; i < n_of_variables; i++)
      if (pwindow [i] != NULL) pwindow [i] ->SetMaxBound (B);
}

int met_out (int n)
{
//fprintf (stderr, "met_out: n - 1 %d fm %p\n", n - 1, *(output .fm));
   output .n_of_points = n - 1;
   if (output .nwindow != NULL) {
      output .nwindow ->SetIteration (n);
      output .nwindow ->SetPoint (output .n_of_variables, (*(output .xn)) [n - 1]);
      output .nwindow ->SetValue ((*(output .fm)) [n - 1]);
   }
   if (output .window != NULL) output .window ->SetN (n);
   for (int i = 0; i < output .n_of_variables; i++) 
      if (output .pwindow [i] != NULL) output .pwindow [i] ->SetN (n);
   output .DisplayItself ();
   for (;;) {
      if (XPending (output .display) == 0) break;

      XEvent event;

      XNextEvent (output .display, &event);

      if (Dispatch (&event) == STOP) {
         for (;;) {
            XEvent event;

            XNextEvent (output .display, &event);
            
            int done = Dispatch (&event);

            switch (done) {
            case RUN:
//fprintf (stderr, "met_out exit\n");
               return NO_EXIT_MET;
            case EXIT:
//fprintf (stderr, "met_out exit\n");
               return EXIT_MET;
            case PROJECTION:
               {
                  int      var   = 1;
                  FELEMENT Fixed [2];

                  Fixed [0] .ident   = VAR;
                  Fixed [0] .pointer = (void *) &var;
                  Fixed [0] .format  = INT;
                  Fixed [1] .pointer = NULL;

                  VELEMENT Variable [1];

                  Variable [0] .pointer = NULL;

                  VAR_DBOX *dbox = new VAR_DBOX (output .display, output .area,
                     DBOX_FOREGROUND, DBOX_BACKGROUND, "var.res", Fixed, 
                     Variable);

                  if (dbox == NULL) {
                     ERROR ("no memory");
                  }
                  for (;;) {
                     dbox ->Dialog ();
                     if (var > 0 && var <= output .n_of_variables) break;
                  }
                  delete dbox;
                  var--;
                  if (output .IsProjectionWindow (var) == False)
                     output .CreateProjectionWindow (var);
                  else
                     output .DestroyProjectionWindow (var);
               }
               break;
            case CONVERGENCE:
               if (output .IsConvergenceWindow () == False)
                  output .CreateConvergenceWindow ();
               else
                  output .DestroyConvergenceWindow ();
               break;
            case NUMERIC:
               if (output .IsNumericWindow () == False)
                  output .CreateNumericWindow (output .n_of_variables);
               else
                  output .DestroyNumericWindow ();
               break;
            }
         }
      }
   }
//fprintf (stderr, "met_out exit\n");
   return NOTHING;
}





