/* File: bayes1.C */

#include <stdio.h>
#include <malloc.h>
#include <values.h>
#include <stdlib.h>
#include <math.h>
#include "const.h"
#include "res.h"
#include "task.h"
#include "ERROR.h"
#include "OUTPUT.h"

#define NOT_SEC_POINT FALSE
#define SEC_POINT     TRUE
#define PAR1          10
#define PAR2          50
#define RMAX          0.5 * r1mach (2)
#define VAL           1e-6
#define COEF          100 * 1e-16

static double *x2 = NULL;
static double *mz;

static void glbayes0 (double *x, const double *a, const double *b, int n, 
   double *y,
   double (*f) (const double *, int), int it, int lt, double *fm, double **xn);
static double fiap1 (double *x, int nn, double **xn, double fmin, double *fm,
   double ff, int m);
static void mig2f2 (double *x, const double *a, const double *b, int nn, 
   double **xn, 
   double *x2, double *z, double *fm, int *ix, double ff, int m);

void bayes1 (double *x, const double *a, const double *b, int n, double *y,
   double (*f) (const double *, int), int it, int lt, double *fm, double **xn)
{
   if ((x2 = (double *) malloc ((unsigned int) sizeof (double) * 2 * n))
      == NULL) {
      ERROR ("out of memory", False);
      goto end;
   }
   mz = x2 + n;
   glbayes0 (x, a, b, n, y, f, it, lt, fm, xn);
end:
   if (x2 != NULL) {
      free ((char *) x2);
      x2 = NULL;
   }
}

static void glbayes0 (double *x, const double *a, const double *b, int n, 
   double *y,
   double (*f) (const double *, int), int it, int lt, double *fm, double **xn)
{
   double ff = r1mach (2);
   int    best_iteration;
   int ix = NOT_SEC_POINT;
   int iteration;
   for (iteration = 0; iteration < lt; iteration++) {
      lp_point (x, a, b, n, iteration);
      fm [iteration] = *y = (*f) (x, n);
      if (*y < ff) {
         ff             = *y;
         best_iteration = iteration;
      }
      copy_array (xn [iteration], x, n);
      if (met_out (iteration + 1) == EXIT_MET) goto end;
   }

   for (iteration = lt; iteration < it; iteration++) {
      mig2f2 (x, a, b, n, xn, x2, mz, fm, &ix, ff, iteration);
      fm [iteration] = *y = (*f) (x, n);
      if (*y < ff) {
         ff             = *y;
         best_iteration = iteration;
      }
      copy_array (xn [iteration], x, n);
      if (met_out (iteration + 1) == EXIT_MET) goto end;
   }

end:
   *y = ff;
   copy_array (x, xn [best_iteration], n);
}

static double fiap1 (double *x, int nn, double **xn, double fmin, double *fm,
   double ff, int m)
{
   double yk, fii;
   int    j;

   yk  = ff - MAX (VAL, COEF * fabs (ff));
   fii = MIN (RMAX, RMAX / (fm [0] - yk));
   for (j = 0; j < m; j++) {
      double dd = distance (xn [j], x, nn);
      double d  = POW_2 (dd);
      double pp = fm [j] - yk;

      if (d < MIN (RMAX, pp * fii)) {
         fii = d / pp;
         if (fii <= -fmin) return (-fii);
      }
   }
   return (-fii);
}

static void mig2f2 (double *x, const double *a, const double *b, int nn, 
   double **xn, 
   double *x2, double *z, double *fm, int *ix, double ff, int m)
{
   int    k, k2;
   double f2;
   double fmin = 0.;

   k2 = MIN (PAR2, MAX (PAR1, m)) * nn;
   if (*ix == NOT_SEC_POINT) {
      random_point (x2, a, b, nn);
      k2--;
      *ix = SEC_POINT;
   }
   f2 = fiap1 (x2, nn, xn, fmin, fm, ff, m);
   for (k = 0; k < k2; k++) {
      double fi, ffm;

      random_point (z, a, b, nn);
      fi = fiap1 (z, nn, xn, fmin, fm, ff, m);
      if (k == 0) {
         if (fi >= f2) {
            copy_array (x, x2, nn);
            copy_array (x2, z, nn);
            ffm  = f2;
            fmin = f2 = fi;
         } else {
            copy_array (x, z, nn);
            ffm  = fi;
            fmin = f2;
         }
      } else {
         if (fi < f2) {
            if (fi < ffm) {
               copy_array (x2, x, nn);
               copy_array (x, z, nn);
               fmin = f2 = ffm;
               ffm  = fi;
            } else {
               copy_array (x2, z, nn);
               fmin = f2 = fi;
            }
         }
      }
   }
}
