/* File: lpmin.C */

#include <stdio.h>
#include <malloc.h>
#include <values.h>
#include <stdlib.h>
#include <math.h>
#include "const.h"
#include "res.h"
#include "task.h"
#include "ERROR.h"
#include "OUTPUT.h"

#define R1M (r1mach (2) / 100.)

static void lpmin0 (double *x, const double *a, const double *b, int n,
   double *y, double (*f) (const double *x, int n), int max_l, 
   int init_points);

/*---------------------------------------------------------------------------*/

static double *fm, *tfm, *xbest;
static double **txn;
static double *x1  = NULL;
static int    *nr  = NULL;
static double **xn = NULL;

void lpmin (double *x, const double *a, const double *b, int n, double *y,
   double (*f) (const double *x, int n), int max_l, int init_points,
   double *pfm, double **pxn)
{
   int i;

   tfm = pfm;
   txn = pxn;
   if ((xn = (double **)
      malloc (init_points * sizeof (double *))) == NULL) {
      ERROR ("out of memory");
      goto ret;
   }
   if ((x1 = (double *)
      malloc ((2 * n + init_points * (n + 1)) *
      sizeof (double))) == NULL) {
      ERROR ("out of memory");
      goto ret;
   }
   if ((nr = (int *) malloc (n * sizeof (int))) == NULL) {
      ERROR ("out of memory");
      goto ret;
   }
   xbest = x1 + n;
   fm    = xbest + n;
   for (i = 0; i < init_points; i++)
      xn [i] = fm + init_points + i * n;
   lpmin0 (x, a, b, n, y, f, max_l, init_points);
ret:
   if (nr != NULL) {
      free ((char *) nr);
      nr = NULL;
   }
   if (x1 != NULL) {
      free ((char *) x1);
      x1 = NULL;
   }
   if (xn != NULL) {
      free ((char *) xn);
      xn = NULL;
   }
}

static void lpmin0 (double *x, const double *a, const double *b, int n,
   double *y, double (*f) (const double *x, int n), int max_l, int init_points)
{
   double ff    = r1mach (2);
   int    iteration, m;
   int    n_of_points = 0;

   for (iteration = 0; iteration < init_points; iteration++) {
      double *px = xn [iteration];

      lp_point (x, a, b, n, iteration);
      scale_to_zero_one_bounds (px, x, a, b, n);
      tfm [n_of_points] = fm [iteration] = *y = (*f) (x, n);
      copy_array (txn [n_of_points], x, n);
      n_of_points++;
      if (fabs (*y) > R1M) {
         ERROR (
"ERROR: function value too large");
         goto end;
      }
      if (*y < ff) {
         ff = *y;
         copy_array (xbest, x, n);
      }
      if (met_out (n_of_points) == EXIT_MET) goto end;
   }
   covariation_matrix_diagonal (xn, fm, x1, init_points, n);
   sort_with_index (x1, nr, n);
   for (m = 0; m < max_l - init_points; m++) {
      int    j;

      lp_point (x1, a, b, n, m);
      for (j = 0; j < n; j++)
         x [nr [j]] = x1 [j];
      tfm [n_of_points] = *y = (*f) (x, n);
      copy_array (txn [n_of_points], x, n);
      n_of_points++;
      if (*y < ff) {
         ff = *y;
         copy_array (xbest, x, n);
      }
      if (met_out (n_of_points) == EXIT_MET) goto end;
   }
end:
   copy_array (x, xbest, n);
   *y = ff;
}
