/* File: test.C */

#include "MENU.h"
#include "CLIENT.h"
#include "const.h"
#include "work.h"
#include "ERROR.h"

static char window_name [] = "Global Minimum";

#define BORDERWIDTH        5
#define BORDER             BlackPixel (display, DefaultScreen (display))
#define BACKGROUND         WhitePixel (display, DefaultScreen (display))
#define FOREGROUND         BORDER

class MAIN_WINDOW : public WINDOW {
   public:
      MAIN_WINDOW (Display *dsp, WINDOW *prnt, char *ClassName, char *Name,
         int x, int y, unsigned int width, unsigned int height,
         unsigned int border_width, unsigned long border,
         unsigned long background, Font fid, unsigned long foreground) :
         WINDOW (dsp, prnt, ClassName, Name, x, y, width, height, border_width,
         border, background, True, fid, foreground) {SetInput (ExposureMask);};
      int Go (XEvent *);
   };

int MAIN_WINDOW::Go (XEvent *event)
{
   switch (event ->type) {
   case Expose:
      if (event ->xexpose .count == 0) {
         XWindowAttributes xwa;

         GetAttributes (&xwa);

         Window       *children;
         unsigned int n_children;

         GetChildren (&children, &n_children);

         for (int i = 0; i < n_children; i++) {
            struct {
               unsigned char *tc;
            } *prop;

            GetThisProperty (display, children [i], (unsigned char **) &prop);

            char *name = ((WINDOW *) prop ->tc) ->ClassName ();

            if (strcmp (name, MENU_NAME) == 0)
               ((MENU *) prop ->tc) ->Resize ();
            else if (strcmp (name, CLIENT_NAME) == 0)
               ((CLIENT *) prop ->tc) ->Resize ();

            XFree ((char *) prop);
         }
         if (n_children > 0) XFree ((char *) children);
      }
      break;
   default:
      break;
   }
   return 0;
}

static char stfont [] = "9x15bold";

int main (int argc, char **argv)
{
   char *font;

   if (argc == 1) {
      font = new char [strlen (stfont) + 1];
      strcpy (font, stfont);
   } else {
      font = new char [strlen (argv [1]) + 1];
      strcpy (font, argv [1]);
   }

   Display     *display   = XOpenDisplay ("");
   
   if (display == NULL) {
      ERROR ("Can't open display");
   }

   XFontStruct *font_info = XLoadQueryFont (display, font);

   if (font_info == NULL) {
      ERROR ("Can't load font");
   }
   delete font;

// create main window

   XSizeHints hint;

   hint .x      = 0;
   hint .y      = 0;
   hint .width  = DisplayWidth (display, DefaultScreen (display)) / 2;
   hint .height = DisplayHeight (display, DefaultScreen (display)) / 2;

   MAIN_WINDOW *window = new MAIN_WINDOW (display, NULL, "MAIN_WINDOW", 
      window_name, hint .x, hint .y, hint .width, hint .height, BORDERWIDTH, 
      BORDER, BACKGROUND, font_info ->fid, FOREGROUND);

   if (window == NULL) {
      ERROR ("no memory");
   }

// create menu

   XSizeHints parent_hint;

   window ->GetNormalHints (&parent_hint);
   hint .x      = -1;
   hint .y      = -1;
   hint .width  = parent_hint .width;
   hint .height = CHAR_HEIGHT;

   MENU *menu = new MENU (display, window, hint .x, hint .y, hint .width,
      hint .height, BACKGROUND, FOREGROUND, "menu.res");

   if (menu == NULL) {
      ERROR ("no memory");
   }

// create client

   XSizeHints menu_hint;

   window ->GetNormalHints (&parent_hint);
   menu ->GetNormalHints (&menu_hint);

   hint .x      = 0;
   hint .y      = menu_hint .y + menu_hint .height + 2;
   hint .width  = parent_hint .width;
   hint .height = parent_hint .height - hint .y;

   CLIENT *client = new CLIENT (display, window, hint .x, hint .y, hint .width,
      hint .height, FOREGROUND, BACKGROUND);

   if (client == NULL) {
      ERROR ("no memory");
   }

   window ->Show (True);
   client ->Run ();
   window ->Show (False);

   delete menu;

   delete client;
   delete window;
   XFreeFont (display, font_info);
   XCloseDisplay (display);
}

