/* File: unt.C */

#include <stdio.h>
#include <malloc.h>
#include <values.h>
#include <stdlib.h>
#include <math.h>
#include "const.h"
#include "res.h"
#include "task.h"
#include "ERROR.h"
#include "OUTPUT.h"

#define I70  70
#define I10  10
#define I40  40
#define I3   3
#define I5   5
#define I7   7
#define HALF 0.5

static void unt0 (double *x, const double *a, const double *b, int n,
   double *y, double (*f) (const double *x, int n), int max_l, 
   int init_points);
static void graunt (double *x, const double *a, const double *b, int n,
   double *y, double s, double zm, double yl, double *sm, double *ds, int *nh,
   const double *am, const double *ab, double *a2 [I5], int kt, 
   int n_of_points,
   double af, const int *nl, const int *nx [I10], int nb, const int *ls,
   double za, double *d, double *a1, const double *ams, double zi, double *z,
   double *ff, double *del);
static double fu (const double *x, int n, double zm, double yl, double *sm,
   double *ds, int *nh,
   const double *am, const double *ab, double *a2 [I5], int kt, 
   int n_of_points,
   double af, const int *nl, const int *nx [I10], int nb, const int *ls,
   double za, double *d, double *a1, const double *ams, double zi);
static void vert (int n, int n_of_points, double *x, int *nh,
   double *am, const double *ab, double *a2 [I5],
   double af, const int *nl, const int *nx [I10], const int *ls,
   double za, double *d, double *a1, double *ams, double zi);
static void smd (double *sm, double *ds, const double *x, int n, int *nh,
   const double *am, const double *ab, double *a2 [I5], int kt, 
   int n_of_points,
   double af, const int *nl, const int *nx [I10], int nb, const int *ls,
   double za, double *d, double *a1, const double *ams, double zi);
static double norm (const double *x, const double *y, int n,
   const double *am);
static double fp (const double *x, int n, 
   double (*f) (const double *x, int n), const double *ab,
   int *nx [I10], int *nl, int n_of_points);

static double **xn;
static double *fm;

static double fp (const double *x, int n, 
   double (*f) (const double *x, int n), const double *ab,
   int *nx [I10], int *nl, int n_of_points)
{
   double y, r;
   int    i, k;

   y        = (*f) (x, n);
   r        = x [0];
   k        = I10 - 1;
   for (i = 1; i < I10; i++)
      if (r < ab [i]) {
         k = i - 1;
         break;
      }
   nx [k] [++(nl [k])] = n_of_points + 1;
   return y;
}

static void vert (int n, int n_of_points, double *x, int *nh,
   double *am, const double *ab, double *a2 [I5],
   double af, const int *nl, const int *nx [I10], const int *ls,
   double za, double *d, double *a1, double *ams, double zi)
/* estimation of scale factors */
{
   int    m, k, kt, nb;
   double f, s, v, fi, sm, ds;

   kt = 1;
   for (k = 0; k < n; k++) {
      f = 0.;
      for (m = 0; m < n_of_points; m++)
         x [m] = xn [m] [k];
      dispersion (x, n_of_points, &f, &s);
      am [k]  = s = n_of_points / (s * (n_of_points - 1));
      ams [k] = sqrt (s);
   }
   *d = 1.;
   s  = v = 0.;
   for (nb = 0; nb < n_of_points; nb++) {
      fi = fm [nb];
      if (fi >= za) continue;
      copy_array (x, xn [nb], n);
      smd (&sm, &ds, x, n, nh, am, ab, a2, kt, n_of_points, af, nl, nx, nb,
         ls, za, d, a1, ams, zi);
      s += ds * (sm - fi) * (sm - fi);
      v += ds * ds;
   }
   *d  = s / v;
}

static double norm (const double *x, const double *y, int n, const double *am)
{
   double r = 0.;

   while (n-- > 0) {
      double t = x [n] - y [n];

      r += am [n] * t * t;
   }
   return (r);
}

static void smd (double *sm, double *ds, const double *x, int n, int *nh,
   const double *am, const double *ab, double *a2 [I5], int kt, 
   int n_of_points,
   double af, const int *nl, const int *nx [I10], int nb, const int *ls,
   double za, double *d, double *a1, const double *ams, double zi)
/* calculates expected value and conditional uncertainity */
{
   double at [I5], an [I5];
   double dz, r2, r3, z, r1, b1, rm, r, s, v, v2, v3;
   int    k1, l1, j, m, i, k, ks, kr, kn, n2;
   double *px1, *pa, *px2, *px;

   if (kt % 2 == 0) {
      for (i = 0; i < I5; i++)
         at [i] = norm (x, xn [nh [i]], n, am);
   } else {
      if (n_of_points > I70) {
         r = x [0];
         k = I10 - 2;
         for (i = 1; i < I10; i++)
            if (r < ab [i]) {
               k = i - 1;
               if (k != 0 && r < ab [k] + af) k--;
               break;
            }
         ks = -1;
         kr = 1;
         kn = nl [k];
         for (i = 0; i < I5; i++) {
            do {
               if (ks >= kn) {
                  ks = -1;
                  kr = 2;
                  kn = nl [++k];
               }
               n2  = nx [k] [++ks];
            } while (n2 == nb);
            nh [i] = n2;
            at [i] = norm (x, xn [n2], n, am);
         }
         array_maximum (at, I5, &m);
         rm = at [m];
         while (kr != 2 || ks != kn) {
            if (ks >= kn) {
               ks = -1;
               kr = 2;
               kn = nl [++k];
            }
            n2  = nx [k] [++ks];
            if (n2 == nb) continue;
            r = norm (x, xn [n2], n, am);
            if (r >= rm) continue;
            nh [m] = n2;
            at [m] = r;
            array_maximum (at, I5, &m);
            rm = at [m];
         }
      } else {
         n2 = 0;
         for (i = 0; i < I5; i++) {
            if (n2 == nb) n2++;
            nh [i] = n2;
            at [i] = norm (x, xn [n2], n, am);
            n2++;
         }
         array_maximum (at, I5, &m);
         rm = at [m];
         for (i = n2; i < n_of_points; i++) {
            if (i == nb) continue;
            r = norm (x, xn [i], n, am);
            if (r >= rm) continue;
            nh [m] = i;
            at [m] = r;
            array_maximum (at, I5, &m);
            rm = at [m];
         }
      }
   }
   for (i = 0; i < I5; i++) {
      r      = at [i];
      rm     = sqrt (r);
      at [i] = rm;
      an [i] = MAX (r1mach (3), exp (-r / 0.3) / (rm + 1.0e-3));
   }
   if (kt < 3) {
      s = v = v2 = v3 = 0.;
      for (j = 0; j < I5; j++) {
         i  = nh [j];
         *ds = 0.;
         if (nb != -1 && fm [i] > za) return;
         r  = at [j];
         b1 = an [j];
         s += b1 * fm [i];
         if (ls [i] < I7) {
            v3 += b1;
            v2 += r * b1;
         } else {
            rm  = r / 5.;
            r1  = MAX (r1mach (3), exp (-rm * rm / 0.3) / (rm + 1.0e-3));
            v3 += r1;
            v2 += r1 * rm;
         }
         v += b1;
      }
      *sm = s / v;
      *ds = *d * v2 / v3;
   } else {
      if (kt == 3) {
         rm = r1mach (3);
         for (i = 0; i < I5; i++) {
            b1  = 0.;
            j   = nh [i];
            z   = fm [j];
            px1 = xn [j];
            pa  = a2 [i];
            for (l1 = 0; l1 < n; l1++) {
               r3 = ams [l1];
               r2 = px1 [l1];
               s  = v = 0.;
               for (k = 0; k < I5; k++) {
                  if (k == i) continue;
                  k1  = nh [k];
                  r1  = an [k];
                  px2 = xn [k1];
                  r   = norm (px1, px2, n, am);
                  v  += r1;
                  s  += (fm [k1] - z) * (px2 [l1] - r2) * r1 * r3 / r;
               }
               pa [l1] = *a1 = s / v;
               b1     += *a1 * *a1;
            }
            rm = MAX (rm, b1);
         }
         *a1 = HALF * (za - zi) / sqrt (rm);
      }
      s = v = 0.;
      for (i = 0; i < I5; i++) {
         r  = at [i];
         j  = nh [i];
         b1 = an [i];
         dz = 0.;
         pa = a2 [i];
         px = xn [j];
         for (k = 0; k < n; k++)
            dz += pa [k] * (x [k] - px [k]) * ams [k];
         v += b1;
         s += b1 * (fm [j] + dz * *a1 / (*a1 + r));
      }
      *sm = s / v;
   }
}

static double fu (const double *x, int n, double zm, double yl, double *sm,
   double *ds, int *nh,
   const double *am, const double *ab, double *a2 [I5], int kt, 
   int n_of_points,
   double af, const int *nl, const int *nx [I10], int nb, const int *ls,
   double za, double *d, double *a1, const double *ams, double zi)
/* function which to be MINimized */
{
   double z;

   if (kt < 3) {
      if (zm < HALF)
         smd (sm, ds, x, n, nh, am, ab, a2, kt, n_of_points, af, nl, nx, nb,
            ls, za, d, a1, ams, zi);
      z   = fabs (zm);
      *ds = MAX (r1mach (3), *ds);
      if (z > HALF) {
         return ((*sm - zi + (za - zi) * z) / sqrt (*ds));
      } else {
         return ((*sm + yl - 2 * zi) / sqrt (*ds));
      }
   } else {
      smd (sm, ds, x, n, nh, am, ab, a2, kt, n_of_points, af, nl, nx, nb,
         ls, za, d, a1, ams, zi);
      return *sm;
   }
}

static void graunt (double *x, const double *a, const double *b, int n,
   double *y, double s, double zm, double yl, double *sm, double *ds, int *nh,
   const double *am, const double *ab, double *a2 [I5], int kt, 
   int n_of_points,
   double af, const int *nl, const int *nx [I10], int nb, const int *ls,
   double za, double *d, double *a1, const double *ams, double zi, double *z,
   double *ff, double *del)
{
   double eps;

   *y  = fu (x, n, zm, yl, sm, ds, nh, am, ab, a2, kt, n_of_points, af, nl,
      nx, nb, ls, za, d, a1, ams, zi);
   eps = 0.03 * s;
   kt++;
   copy_array (z, x, n);
   for (;;) {
      double sg;
      int    i;

      for (i = 0; i < n; i++) {
         double x1, f1, f2, d1;

         x1 = x [i];
         if (x1 > b [i] - eps || x1 < a [i] + eps) {
            ff [i] = 0.;
            continue;
         }
         d1     = del [i];
         z [i]  = x1 + d1;
         f1     = fu (z, n, zm, yl, sm, ds, nh, am, ab, a2, kt, n_of_points,
            af, nl, nx, nb, ls, za, d, a1, ams, zi);
         z [i]  = x1 - d1;
         f2     = fu (z, n, zm, yl, sm, ds, nh, am, ab, a2, kt, n_of_points,
            af, nl, nx, nb, ls, za, d, a1, ams, zi);
         ff [i] = (f1 - f2) / (2 * d1);
         z [i]  = x1;
      }
      sg  = norma (ff, n);
      if (sg < r1mach (3)) return;
      for (;;) {
         double dx1 = s / sg;
         double f1;

         for (i = 0; i < n; i++) {
            double dx = dx1 * ff [i];

            z [i] = increment (x [i], a [i], b [i], - dx);
         }
         kt--;
         f1     = fu (z, n, zm, yl, sm, ds, nh, am, ab, a2, kt, n_of_points,
            af, nl, nx, nb, ls, za, d, a1, ams, zi);
         kt++;
         if (f1 < *y) {
            *y = f1;
            copy_array (x, z, n);
            break;
         } else {
            s *= HALF;
         }
         if (s < eps) return;
      }
   }
}

/*---------------------------------------------------------------------------*/

#define K5   5   /* Number of local minima */
#define I100 100

static int    *ls = NULL;
static int    ks [K5];
static int    *nx [I10];
static double *am = NULL;
static double *ams, *graff, *del, *z, *work, *fs, *x1, *x2;
static double *xs [K5];
static double *a2 [I5];
static int    nh [I5], nl [I10];
static double ab [I10];

void unt (double *x, const double *a, const double *b, int n, double *y,
   double (*f) (const double *x, int n), int max_l, int init_points, 
   double *pfm, double **pxn)
{
   fm = pfm;
   xn = pxn;

   int k        = MAX (K5, MIN (init_points / I5, I10));
   int i;

   if ((ls = (int *) malloc ((max_l + I10 * 100) * sizeof (int)))
      == NULL) {
      ERROR ("out of memory", False);
      goto end;
   }
   for (i = 0; i < I10; i++) nx [i] = ls + max_l + i * I100;
   if ((am = (double *) malloc (
      (7 * n + init_points + k + K5 * n + I5 * n) * sizeof (double))) 
      == NULL) {
      ERROR ("out of memory", False);
      goto end;
   }
   ams   = am + n;
   graff = ams + n;
   del   = graff + n;
   z     = del + n;
   work  = z + n;
   x1    = work + init_points;
   x2    = x1 + n;
   fs    = x2 + n;
   for (i = 0; i < K5; i++) xs [i] = fs + k + i * n;
   for (i = 0; i < I5; i++) a2 [i] = xs [K5 - 1] + n + i * n;
   unt0 (x, a, b, n, y, f, max_l, init_points);
end:
   if (am != NULL) {
      free ((char *) am);
      am = NULL;
   }
   if (ls != NULL) {
      free ((char *) ls);
      ls = NULL;
   }
}

static void unt0 (double *x, const double *a, const double *b, int n,
   double *y, double (*f) (const double *x, int n), int max_l, int init_points)
{
   int    n_of_points, kau, i, k, n1, kt, kx, k4, k2, nb, m, kxl;
   double s1, s2, af, zi, za, fi, d, a1, yl, xl, zm, sm, ds, d1, r;

   n_of_points = 0;
   kau         = 0;
   integer_array_fill (ls, max_l, 0);
   s2 = distance (a, b, n) * HALF;
   for (i = 0; i < n; i++)
      del [i] = (b [i] - a [i]) * 1.0e-4;
   af = (b [0] - a [0]) / I10;
   integer_array_fill (nl, I10, -1);
   for (i = 0; i < I10; i++)
      ab [i] = a [0] + i * af;
   zi = r1mach (2);
   for (i = 0; i < init_points; i++) {
      int    m     = i % I10;
      double fi    = ab [m] + af;

      random_point (z, ab + m, &fi, 1);
      random_point (z + 1, a + 1, b + 1, n - 1);
      fm [n_of_points] = *y = fp (z, n, f, ab, nx, nl, n_of_points);
      copy_array (xn [n_of_points], z, n);
      n_of_points++;
      if (*y < zi) {
         zi = *y;
         copy_array (x, z, n);
      }
      if (met_out (n_of_points) == EXIT_MET) goto end;
   }
/* estimation of k-th MAXimal function value za */
   af *= HALF;
   k   = MIN (init_points / I5, I10);
   for (i = 0; i < k; i++) fs [i] = fm [i];
   array_minimum (fs, k, &n1);
   za = fs [n1];
   for (i = k; i < init_points; i++) {
      fi = fm [i];
      if (fi > za) {
         fs [n1] = fi;
         array_minimum (fs, k, &n1);
         za = fs [n1];
      }
   }
   fi = fabs (zi - za);
   if (fi < r1mach (3) || fi > sqrt (r1mach (2))) {
      ERROR (
"variation of objective function is too small or too large", False);
      goto end;
   }
   kt = 1;
   vert (n, init_points, work, nh, am, ab, a2, af, nl, nx, ls,
      za, &d, &a1, ams, zi);
   nb = -1;
   yl = 0.7 * zi + 0.3 * za;
   kx = 0;
   k4 = I3;
   xl = 0.;
   k2 = (max_l + init_points) / 2;
/* main loop of global MINimization */
   for (i = init_points; i < k2; i++) {
      double f1, f2, f3, f4;

      kt = 1;
      f3 = f4 = r1mach (2);
      for (m = 0; m < I40; m++) {
         random_point (z, a, b, n);
         zm = 0.1;
         f1 = fu (z, n, zm, yl, &sm, &ds, nh, am, ab, a2, kt, n_of_points,
            af, nl, nx, nb, ls, za, &d, &a1, ams, zi);
         zm = 1.;
         f2 = fu (z, n, zm, yl, &sm, &ds, nh, am, ab, a2, kt, n_of_points,
            af, nl, nx, nb, ls, za, &d, &a1, ams, zi);
         if (f1 < f3) {
            f3 = f1;
            copy_array (x2, z, n);
         }
         if (f2 < f4) {
            f4 = f2;
            copy_array (x1, z, n);
         }
      }
      zm = -1.;
      kt = 1;
      s1 = s2 / pow ((double) n_of_points, 1. / n);
      graunt (x1, a, b, n, &fi, s1, zm, yl, &sm, &ds, nh, am, ab, a2, kt,
         n_of_points, af, nl, nx, nb, ls, za, &d, &a1, ams, zi, work, graff,
         del);
      fm [n_of_points] = f1 = fp (x1, n, f, ab, nx, nl, n_of_points);
      copy_array (xn [n_of_points], x1, n);
      n_of_points++;
      if (f1 < zi) {
         zi = f1;
         copy_array (x, x1, n);
      }
      if (met_out (n_of_points) == EXIT_MET) goto end;
      zm = -0.1;
      kt = 1;
      s1 = s2 / pow ((double) n_of_points, 1. / n);
      graunt (x2, a, b, n, &fi, s1, zm, yl, &sm, &ds, nh, am, ab, a2, kt,
         n_of_points, af, nl, nx, nb, ls, za, &d, &a1, ams, zi, work, graff,
         del);
      d1  = distance (x1, x2, n);
      kxl = 0;
      if (d1 <= 0.01 * s2) {
         kxl = 1;
         random_point (x2, a, b, n);
      }
      fm [n_of_points] = f2 = fp (x2, n, f, ab, nx, nl, n_of_points);
      copy_array (xn [n_of_points], x2, n);
      n_of_points++;
      if (f2 < zi) {
         zi = f2;
         copy_array (x, x2, n);
      }
      if (met_out (n_of_points) == EXIT_MET) goto end;
      if (kxl == 1) {
         f2 = f1;
         copy_array (x2, x1, n);
      }
      kx++;
      xl = ((kx - 1.) * xl + f2) / kx;
      if (k % I10 == I10 - 1) yl = xl;
      n1 = -1;
      for (m = 0; m < I5; m++) {
         i      = nh [m];
         if (++(ls [i]) == I7) {
            fi = fm [i];
            if (n1 == -1) {
               f3 = fi;
               n1 = i;
            } else {
               if (fi < f3) {
                  f3 = fi;
                  n1 = i;
               }
            }
         }
      }
      if (n1 == -1) continue;
      if (f2 > f3) {
         copy_array (z, xn [n1], n);
      } else {
         copy_array (z, x2, n);
      }
/* estimation of local MINimum of fu */
      kt = 3;
      graunt (z, a, b, n, &fi, s1, zm, yl, &sm, &ds, nh, am, ab, a2, kt,
         n_of_points, af, nl, nx, nb, ls, za, &d, &a1, ams, zi, work, graff,
         del);
      n1      = nh [0];
      fi      = fm [n1];
      ls [n1] = I7;
      for (m = 1; m < I5; m++) {
         i = nh [m];
         r = fm [i];
         if (r < fi) {
            n1 = i;
            fi = r;
         }
         ls [i] = I7;
      }
      copy_array (z, xn [n1], n);
      if (kau != 0) {
         for (i = 0; i < kau; i++) {
            r = norm (xs [i], x1, n, am);
            if (r >= 0.04) continue;
            r = fs [i];
            if (fi < r) {
               copy_array (xs [i], z, n);
               fs [i] = fi;
            }
            n1 = ks [i];
            if (n1 >= k4) {
               ERROR (
"density of points in local MINimum area equals 3", False);
               goto end;
            }
            ks [i] = n1 + 1;
            break;
         }
         if (n1 < k4) continue;
      }
      ks [kau] = 1;
      fs [kau] = fi;
      copy_array (xs [kau], z, n);
      if (++kau >= K5) {
         ERROR (
"number of local minimuma equals 5", False);
         goto end;
      }
   }
end:
   *y = zi;
}

